@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+ * Copyright 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
  * Written by Paolo Bonzini.  Ideas based on Mike Haertel's malloc.
  *
  * This file is part of GNU Smalltalk.
@@ -97,11 +97,6 @@
 
 static void init_heap (heap_data *h, size_t heap_allocation_size, size_t heap_limit);
 
-/* Oh my God, how broken things sometimes are!  */
-#if defined small
-# undef small
-#endif
-
 #define vSmall var.small
 #define vLarge var.large
 #define vFree  var.free
