@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -331,22 +331,6 @@ get_errno (void)
   return (old);
 }
 
-int
-my_utime (const char *name,
-	  long new_atime,
-	  long new_mtime)
-{
-  struct timeval times[2];
-  int result;
-  times[0].tv_sec = new_atime + 86400 * 10957;
-  times[1].tv_sec = new_mtime + 86400 * 10957;
-  times[0].tv_usec = times[1].tv_usec = 0;
-  result = utimes (name, times);
-  if (!result)
-    errno = 0;
-  return (result);
-}
-
 int
 my_stat (const char *name,
 	 gst_stat * out)
@@ -539,7 +523,7 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("strerror", strerror);
   _gst_define_cfunc ("stat", my_stat);
   _gst_define_cfunc ("lstat", my_lstat);
-  _gst_define_cfunc ("utime", my_utime);
+  _gst_define_cfunc ("utime", _gst_set_file_access_times);
 
   _gst_define_cfunc ("opendir", my_opendir);
   _gst_define_cfunc ("closedir", closedir);
@@ -547,6 +531,7 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("rewinddir", rewinddir);
   _gst_define_cfunc ("extractDirentName", extract_dirent_name);
 
+  _gst_define_cfunc ("symlink", symlink);
   _gst_define_cfunc ("unlink", unlink);
   _gst_define_cfunc ("rename", rename);
   _gst_define_cfunc ("rmdir", rmdir);
