@@ -7,7 +7,8 @@
 
 /***********************************************************************
  *
- * Copyright 2000, 2001, 2002, 2003, 2005, 2006 Free Software Foundation, Inc.
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007
+ * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
  * This file is part of GNU Smalltalk.
@@ -1088,6 +1089,14 @@ init_c_symbols ()
 #define NAN (0.0 / 0.0)
 #endif
 
+#if defined WIN32 && !defined __CYGWIN__
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("PathSeparator"),
+		    CHAR_OOP_AT ('\\'));
+#else
+  NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("PathSeparator"),
+		    CHAR_OOP_AT ('/'));
+#endif
+
   NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMin"),
 		    floatd_new (DBL_MIN));
   NAMESPACE_AT_PUT (cSymbolsOOP, _gst_intern_string ("CDoubleMax"),
@@ -1530,9 +1539,9 @@ _gst_grow_dictionary (OOP oldDictionaryOOP)
      object, the new dictionary itself */
 
   dictionary = instantiate_with (OOP_CLASS (oldDictionaryOOP), 
-						  numFields, &dictionaryOOP);
+				 numFields, &dictionaryOOP);
   memcpy (dictionary->data, oldDictionary->data, sizeof (PTR) * numFixedFields);
-  oldDictionary = OOP_TO_OBJ (dictionaryOOP);
+  oldDictionary = OOP_TO_OBJ (oldDictionaryOOP);
 
   /* rehash all associations from old dictionary into new one */
   for (i = 0; i < oldNumFields; i++)
