@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -18,6 +18,28 @@
  * Software Foundation; either version 2, or (at your option) any later 
  * version.
  * 
+ * Linking GNU Smalltalk statically or dynamically with other modules is
+ * making a combined work based on GNU Smalltalk.  Thus, the terms and
+ * conditions of the GNU General Public License cover the whole
+ * combination.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * give you permission to combine GNU Smalltalk with free software
+ * programs or libraries that are released under the GNU LGPL and with
+ * independent programs running under the GNU Smalltalk virtual machine.
+ *
+ * You may copy and distribute such a system following the terms of the
+ * GNU GPL for GNU Smalltalk and the licenses of the other code
+ * concerned, provided that you include the source code of that other
+ * code when and as the GNU GPL requires distribution of source code.
+ *
+ * Note that people who make modified versions of GNU Smalltalk are not
+ * obligated to grant this special exception for their modified
+ * versions; it is their choice whether to do so.  The GNU General
+ * Public License gives permission to release a modified version without
+ * this exception; this exception also makes it possible to release a
+ * modified version which carries forward this exception.
+ *
  * GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
@@ -34,6 +56,10 @@
 #include "snprintfv/mem.h"
 
 
+/* Print a Character OOP to a snprintfv stream, STREAM.  */
+static void print_char_to_stream (STREAM *stream,
+				  OOP oop);
+
 /* Print a String OOP to a snprintfv stream, STREAM.  */
 static void print_string_to_stream (STREAM *stream,
 				    OOP string);
@@ -71,6 +97,21 @@ _gst_print_object (OOP oop)
   fflush (stdout);
 }
 
+void
+print_char_to_stream (STREAM *stream, OOP oop)
+{
+  int val = CHAR_OOP_VALUE (oop);
+  if (OOP_CLASS (oop) == _gst_char_class && val > 127)
+    stream_printf (stream, "Character value: 16r%02X", val);
+
+  else if (val >= 32 && val <= 126)
+    stream_printf (stream, "$%c", val);
+  else if (val < 32)
+    stream_printf (stream, "$<%d>", val);
+  else
+    stream_printf (stream, "$<16r%04X>", val);
+}
+
 void
 print_string_to_stream (STREAM *stream, OOP string)
 {
@@ -167,8 +208,9 @@ printf_oop (STREAM *stream,
   else if (oop == _gst_false_oop)
     stream_printf (stream, "false");
 
-  else if (OOP_CLASS (oop) == _gst_char_class)
-    stream_printf (stream, "$%c", CHAR_OOP_VALUE (oop));
+  else if (OOP_CLASS (oop) == _gst_char_class
+	   || OOP_CLASS (oop) == _gst_unicode_character_class)
+    print_char_to_stream (stream, oop);
 
   else if (OOP_CLASS (oop) == _gst_floatd_class)
     {
@@ -335,7 +377,7 @@ _gst_display_oop (OOP oop)
 }
 
 void
-_gst_display_object (mst_Object obj)
+_gst_display_object (gst_object obj)
 {
   if (IS_OOP_ADDR (obj))
     {
