@@ -11,7 +11,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2006,2007
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -58,6 +58,10 @@
 
 #include "gstpriv.h"
 
+#ifdef HAVE_UTIME_H
+# include <utime.h>
+#endif
+
 #ifdef HAVE_SYS_TIMES_H
 # include <sys/times.h>
 #endif
@@ -745,30 +749,56 @@ _gst_get_cur_dir_name (void)
 }
 
 
+int
+_gst_set_file_access_times (const char *name, long new_atime, long new_mtime)
+{
+  int result;
+#if defined HAVE_UTIMES
+  struct timeval times[2];
+  times[0].tv_sec = new_atime + 86400 * 10957;
+  times[1].tv_sec = new_mtime + 86400 * 10957;
+  times[0].tv_usec = times[1].tv_usec = 0;
+  result = utimes (name, times);
+#elif defined HAVE_UTIME
+  struct utimbuf utb;
+  utb.actime = new_atime + 86400 * 10957;
+  utb.modtime = new_mtime + 86400 * 10957;
+  result = utime (name, &utb);
+#else
+#warning neither utime nor utimes are available.
+  errno = ENOSYS;
+  result = -1;
+#endif
+  if (!result)
+    errno = 0;
+  return (result);
+}
+
+
 char *
 _gst_get_full_file_name (const char *fileName)
 {
   char *fullFileName;
   static char *fullPath = NULL;
 
-  if (fileName[0] == '/')	/* absolute, so don't need to change */
+  /* Absolute paths don't need to change */
+  if (fileName[0] == '/')
     return (xstrdup (fileName));
+#if defined WIN32 && !defined __CYGWIN__
+  if (fileName[0] == '\\' || (fileName[0] != '\0' && fileName[1] == ':'))
+    return (xstrdup (fileName));
+#endif
 
+  /* Only need to do this once, then cache the result */
   if (fullPath == NULL)
-    {
-      /* Only need to do this once, then cache the result */
-      fullPath = _gst_get_cur_dir_name ();
-    }
+    fullPath = _gst_get_cur_dir_name ();
 
-  /* 
-   * ### canonicalize filename and full path here in the future (remove any
-   * extraneous .. or . directories, etc.)
-   */
+#if !defined WIN32 || defined __CYGWIN__
+  asprintf (&fullFileName, "%s/%s", fullPath, fileName);
+#else
+  asprintf (&fullFileName, "%s\\%s", fullPath, fileName);
+#endif
 
-  fullFileName = (char *) xmalloc (strlen (fullPath) + strlen (fileName) + 1	/* slash 
-										 */
-				   + 1 /* trailing nul */ );
-  sprintf (fullFileName, "%s/%s", fullPath, fileName);
   return (fullFileName);
 }
 
@@ -806,6 +836,9 @@ _gst_file_is_executable (const char *fileName)
 }
 
 
+/* Code to use PTY's did not work on Mac OS.  I'm keeping the Unix code,
+   but it should not be used.  */
+#ifdef __MSVCRT__
 typedef struct pty_info {
   int access;
   int master;
@@ -907,7 +940,7 @@ open_pty (pty_info *pty)
 		      return pty->master;
 		    }
 
-		  pty->slavenam[strlen ("/dev/")] = 'p';
+		  slavenam[strlen ("/dev/")] = 'p';
 		  close (pty->master);
 		}		/* open master */
 	    }			/* hex digit */
@@ -1170,6 +1203,118 @@ _gst_open_pipe (const char *command,
   else
     return (master);
 }
+#else
+
+/* Use sockets or pipes.  */
+int
+_gst_open_pipe (const char *command,
+		const char *mode)
+{
+  int fd[2];
+  int our_fd, child_fd;
+  int master;
+  int access;
+  int result;
+
+  access = strchr (mode, '+') ? O_RDWR :
+    (mode[0] == 'r' ? O_RDONLY : O_WRONLY);
+
+  if (access == O_RDWR)
+    {
+      result = socketpair (AF_UNIX, SOCK_STREAM, 0, fd);
+      our_fd = fd[1];
+      child_fd = fd[0];
+    }
+  else
+    {
+      result = pipe (fd);
+      our_fd = access == O_RDONLY ? fd[0] : fd[1];
+      child_fd = access == O_RDONLY ? fd[1] : fd[0];
+    }
+
+  if (result == -1)
+    return -1;
+
+  _gst_set_signal_handler (SIGPIPE, SIG_DFL);
+
+#ifdef HAVE_SPAWNL
+  {
+    /* Prepare file descriptors, saving the old ones so that we can keep
+       them.  */
+    int save_stdin = -1, save_stdout = -1, save_stderr = -1;
+    if (access != O_WRONLY)
+      {
+        save_stdout = dup (1);
+        save_stderr = dup (2);
+        dup2 (child_fd, 1);
+        dup2 (child_fd, 2);
+      }
+    if (access != O_RDONLY)
+      {
+        save_stdin = dup (0);
+        dup2 (child_fd, 0);
+      }
+
+#ifdef __CYGWIN__
+    result = spawnl (P_NOWAIT, "/bin/sh", "/bin/sh", "-c", command, NULL);
+#else
+    result = spawnl (P_NOWAIT, getenv("COMSPEC"), "/C", command, NULL);
+#endif
+
+    if (save_stdin != -1)
+      {
+        dup2 (save_stdin, 0);
+        close (save_stdin);
+      }
+
+    if (save_stdout != -1)
+      {
+        dup2 (save_stdout, 1);
+        close (save_stdout);
+      }
+
+    if (save_stderr != -1)
+      {
+        dup2 (save_stderr, 2);
+        close (save_stderr);
+      }
+  }
+#else /* !HAVE_SPAWNL */
+  /* We suppose it is a system that has fork.  */
+  result = fork ();
+  if (result == 0)
+    {
+      /* Child process */
+      close (our_fd);
+      if (access != O_WRONLY)
+        {
+          dup2 (child_fd, 1);
+          dup2 (child_fd, 2);
+        }
+      if (access != O_RDONLY)
+        dup2 (child_fd, 0);
+
+      _exit (system (command) >= 0);
+      /*NOTREACHED*/
+    }
+
+#endif /* !HAVE_SPAWNL */
+
+  close (child_fd);
+  _gst_set_signal_handler (SIGPIPE, SIG_IGN);
+
+  if (result == -1)
+    {
+      int save_errno;
+      save_errno = errno;
+      close (our_fd);
+      errno = save_errno;
+      return (-1);
+    }
+  else
+    return (our_fd);
+}
+#endif
 
 int
 _gst_open_file (const char *filename,
@@ -1249,7 +1394,7 @@ _gst_recv (int fd,
 
   for (;;)
     {
-      result = recv (fd, buffer, size, flags);
+      result = recvfrom (fd, buffer, size, flags, NULL, NULL);
       if (errno == EFAULT)
         abort ();
 
@@ -1658,7 +1803,7 @@ anon_mmap_commit (PTR base, size_t size)
 		          PROT_READ | PROT_WRITE | PROT_EXEC,
 		          MAP_PRIVATE | MAP_FIXED);
 
-  return result;
+  return UNCOMMON (result == MAP_FAILED) ? NULL : result;
 }
 
 /* This is hairy and a hack.  We have to find a place for our heaps...  */
